//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

#include <tigcclib.h>
#include <extgraph.h>

//-----------------------------------------------------------------------------

void MAT_Init(void);
void MAT_Exchange(int type, int d1, int d2);
void MAT_Shuffle(void);

void XorTile(int t);
void Shuffle(void);
void NewGame(void);
int HasWon(void);

void DrawScreen(void);

//-----------------------------------------------------------------------------

int matrix[16][16];
int TilesState[16];

int curpos=0;
int xpos=0;
int ypos=0;

//-----------------------------------------------------------------------------

unsigned short CursorData[16]=
{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,};


unsigned short Tiles_Data[64] =
{
    0x07C0, 0x18F0, 0x2BB8, 0x583C, 0x77DC, 0x8C66, 0xA8AE, 0xA86A, 
    0xEAEA, 0xCDE2, 0x77D4, 0x7824, 0x3B88, 0x1E30, 0x07C0, 0x0000, 
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 
    0xFFFF, 0xF83F, 0xE7CF, 0xCAA7, 0xD7D7, 0xACEB, 0xBBFB, 0xAFEF, 
    0xBFFF, 0xAFEF, 0xD7DF, 0xCABF, 0xE7FF, 0xF9FF, 0xFFFF, 0xFFFF, 
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 
    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 
} ;

//-----------------------------------------------------------------------------

char* Plane0;
char* Plane1;

//-----------------------------------------------------------------------------

void MAT_Init(void)
{
	int x,y;
	
	for (y=0;y<16;y++)
	for (x=0;x<16;x++)
	{
		matrix[y][x]=((x+y)&15);
	}
	
}
//-----------------------------------------------------------------------------

void MAT_Exchange(int type, int d1, int d2)
{
	int x,y;
	int temp;
	
	if (type)
	{
		// EXCHANGE COLUMNS D1 & D2
		
		for (y=0;y<16;y++)
		{
			temp=matrix[y][d1];
			matrix[y][d1]=matrix[y][d2];
			matrix[y][d2]=temp;
		}
	}
	else
	{
		
		// EXCHANGE ROWS D1 & D2
		
		for (x=0;x<16;x++)
		{
			temp=matrix[d1][x];
			matrix[d1][x]=matrix[d2][x];
			matrix[d2][x]=temp;
		}
		
	}
	
	
}
//-----------------------------------------------------------------------------

void MAT_Shuffle(void)
{
	int a;
	int r1,r2;
	
	// exchange columns & rows
	
	for (a=0;a<32;a++)
	{
		r1=random(16);
		
		do
		{
			r2=random(16);
		}
		while (r1==r2);
		
		MAT_Exchange(random(2),r1,r2);
	}
	
}
//-----------------------------------------------------------------------------

void XorTile(int t)
{
	int a;
	
	// get corresponding row
	int row=0;
	while (matrix[row++][0]!=t);
	row--;
	
	// num of linked tiles = 4
	for (a=0;a<4;a++)
	TilesState[matrix[row][a]]^=1;
	
}
//-----------------------------------------------------------------------------

void Shuffle(void)
{
	int a,t;
	
	// reset
	for (t=0;t<16;t++)
	TilesState[t]=0;
	
	// shuffle
	for (a=0;a<32;a++)
	XorTile(random(16));
}
//-----------------------------------------------------------------------------

void NewGame(void)
{
	MAT_Init();
	MAT_Shuffle();
	
	Shuffle();
	
	// cursor pos
	xpos=0;
	ypos=0;
	
}
//-----------------------------------------------------------------------------

int HasWon(void)
{
	int t;
	
	for (t=0;t<16;t++)
	if (TilesState[t]!=0)
	return 0;
	
	return 1;
}
//-----------------------------------------------------------------------------

void DrawScreen(void)
{
	int x,y;
	
	GrayClearScreen2B(Plane0,Plane1);
	
	for (y=0;y<4;y++)
	for (x=0;x<4;x++)
	{
		int t=(y<<2)+x;
		int xx=48+(x<<4);
		int yy=18+(y<<4);
		
		//char ch=(TilesState[t]?'1':'0');
		//DrawChar(xx+5,yy+4,ch,A_NORMAL);
		
		unsigned short*src=Tiles_Data+(TilesState[t]?16:0);
		GraySprite16_OR(xx,yy,16,src,src+32,Plane0,Plane1);
		
		
	}
	
	// cursor
	int xc=48+(xpos<<4);
	int yc=18+(ypos<<4);
	GraySprite16_XOR(xc,yc,16,CursorData,CursorData,Plane0,Plane1);
	
	// title
	GrayDrawStr2B(59,5,"XORMAZE",A_NORMAL,Plane0,Plane1);
	
	// win state
	if (HasWon())
	GrayDrawStr2B(56,87,"YOU WIN!",A_NORMAL,Plane0,Plane1);
	
	
	FastCopyScreen(Plane0,GetPlane(0));
	FastCopyScreen(Plane1,GetPlane(1));
	
	
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	
	clrscr();
	FontSetSys(0);
	printf("XorMaze 1.0 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	Plane0=malloc(2*LCD_SIZE);
	Plane1=Plane0+LCD_SIZE;
	
	randomize();
	
	NewGame();
	
	GrayOn();
	
	for (;;)
	{
		xpos&=3;
		ypos&=3;
		curpos=(ypos<<2)+xpos;
		
		DrawScreen();
		
		int key=ngetchx();
		
		if (key==KEY_F1)
		XorTile(curpos);
		else if (key==KEY_F2)
		NewGame();
		
		else if (key==KEY_LEFT)
		xpos--;
		else if (key==KEY_RIGHT)
		xpos++;
		else if (key==KEY_UP)
		ypos--;
		else if (key==KEY_DOWN)
		ypos++;
		
		else if (key==KEY_APPS)
		off();
		else if (key==KEY_ESC)
		break;
		
	}
	
	GrayOff();
	
	if (Plane0)
	free(Plane0);
	
}
